#include "pudag.hh"
#include <utility>
#include <boost/graph/adjacency_iterator.hpp>

lcc_node_map::type lcc_node_property(DAG & _dag)
{
	lcc_node_t pname;
	return get(pname, _dag);
}

time_stamp_map::type time_stamp_property(DAG & _dag)
{
	time_stamp_t pname;
	return get(pname, _dag);
}

lcc::Node get_lcc_node(vertex_id _v, DAG & _dag)
{
	return get(lcc_node_property(_dag), _v);
}

int get_time_stamp(vertex_id _v, DAG & _dag)
{
	return get(time_stamp_property(_dag), _v);	
} 

edge_type_map::type edge_type_property(DAG & _dag)
{
	edge_type_t pname;
	return get(pname, _dag);
}

enum EDGE_T get_edge_type(edge_id _e, DAG & _dag)
{
	return get(edge_type_property(_dag), _e);
}

const char *edgetype(enum EDGE_T _et) {
	const char *edge_type[] = {
		"KID0",
		"KID1",
		"LINK",
		"DDEP"
	};
	return edge_type[_et];
}

vertex_id add_vertex(lcc::Node _p, int _time_stamp, DAG & _dag)
{
	lcc_node_t lcc_pmap;
	time_stamp_t ts_pmap;

	vertex_id u = add_vertex(_dag);
	put(lcc_pmap, _dag, u, _p);
	put(ts_pmap, _dag, u, _time_stamp);
	return u;
}

//Add directed edge _u --> _v
edge_id add_edge(vertex_id _u, vertex_id _v, enum EDGE_T _et, DAG & _dag)
{
	edge_type_t et_pmap;
	std::pair<edge_id, bool> e_desc = add_edge(_u, _v, _dag);
	edge_id e = e_desc.first;
	put(et_pmap, _dag, e, _et);
	return e;
}

std::pair< predecessor_iterator, predecessor_iterator >
predecessors(vertex_id _v_id, DAG & _dag)
{
  graph_traits< DAG >::in_edge_iterator ei, ei_end;
  tie(ei, ei_end) = in_edges(_v_id , _dag);
  predecessor_iterator pi(ei, & _dag), pi_end(ei_end, & _dag);
  return std::make_pair(pi, pi_end);
}

std::pair< successor_iterator, successor_iterator >
successors(vertex_id _v_id, DAG & _dag)
{
  graph_traits< DAG >::out_edge_iterator ei, ei_end;
  tie(ei, ei_end) = out_edges(_v_id , _dag);
  successor_iterator pi(ei, & _dag), pi_end(ei_end, & _dag);
  return std::make_pair(pi, pi_end);
}
